/*
 * File  : DTPA-UART-1616S_ex.c:
 * version : 1.0.0
 * Date : 2023. 4. 19.
 * Contact : expoeb2@diwell.com 
 * Corporation : DIWELL Electronics
 *
 * description : 
 *   DTPA-UART-1616S example code.  
 *   (using wiringPi Library)
 *
 * Further information:
 *   https://projects.drogon.net/raspberry-pi/wiringpi/
 *
 ***********************************************************************
 * Development environment(hardware):
 *   Raspberry Pi 2 Model B v1.1
 ***********************************************************************
 * Revision history.
 * 1. 2023.4.19. : First version is released.
 ***********************************************************************
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <wiringPi.h>
#include <wiringSerial.h>
#include "desc.h"

/*  Define the size of the incoming data */
#define START		2		// 2byte
#define END		2		// 2byte
#define NUM_PIXEL	256		//
#define BUF_SIZE 	(START+END+NUM_PIXEL*2)	// RX varaiale size

/* Define the timing */
#define REQUEST_CYCLE 	100		// 100ms : Request cycle. >100ms
#define TIMEOUT		70		// 66ms : fixed
#define TRANSMISSION_TIME	5	// 5ms : fixed

query_t telegram;			// Master query structure

uint8_t *ptr, com_result;
uint8_t RX_BUF[BUF_SIZE];
int16_t a16data[(BUF_SIZE>>1)-START-END];
uint16_t Request_interval = REQUEST_CYCLE-TIMEOUT-TRANSMISSION_TIME; // 100ms-70ms-5ms = 25ms
int32_t fd;
uint32_t Start_ms;

int main ()
{	
   	telegram.u8start = 0x11;                        // Start : 0x11
   	telegram.u16RegAdd = 1;                         // Pixel 0 start address : 1 
	telegram.u16RegNo = NUM_PIXEL;                  // number of elements to read : 256 pixel
	telegram.u8end = 0x98;				// End : 0x98
	telegram.au16reg = a16data;                     // pointer to a memory array in the raspberry 

	if ((fd = serialOpen ("/dev/ttyAMA0", 115200)) < 0)
	{
		fprintf (stderr, "Unable to open serial device: %s\n", strerror (errno)) ;
		return 1 ;
	}
	if (wiringPiSetup () == -1)
	{
		fprintf (stdout, "Unable to start wiringPi: %s\n", strerror (errno)) ;
		return 1 ;
	}
	
        delay(1000);               			// waiting for Sensor init.   			

	while(1)  					
 	{
		com_result = query(telegram);		// communication.

    		if(com_result == COM_OK)
		{
			printf("pixel Temperature: \n");
			printarray(0, a16data, NUM_PIXEL);	// print pixel temperature
		}
		else if(com_result == COM_ERROR)
		{
			printf("COM Error.\n");
		}
		else if(com_result == NO_REPLY)
		{
			printf("Timeout.\n");
		}
		
		Start_ms = millis();
		while( millis()-Start_ms < Request_interval); 		// Delay : next request

		
 	}
 	//printf ("================Stopped================\n") ;
  	return 0 ;
}

int8_t query(query_t telegram)
{
	uint8_t au8Buffer[6], tx_count,response_status = NO_REPLY;
	uint16_t i,rx_count=0, crc16, *au16regs;

	au16regs = telegram.au16reg;
 	au8Buffer[ 0 ]  = telegram.u8start;
    	au8Buffer[ 1 ]  = highByte(telegram.u16RegAdd );
    	au8Buffer[ 2 ]  = lowByte( telegram.u16RegAdd );
	au8Buffer[ 3 ]  = highByte(telegram.u16RegNo );
        au8Buffer[ 4 ]  = lowByte( telegram.u16RegNo );
	au8Buffer[ 5 ]  = telegram.u8end;
        

	fflush (stdout) ;			// flush buffer
	ptr = au8Buffer;
	
	for(tx_count=0; tx_count<6; tx_count++)
	{
		serialPutchar (fd, *ptr++) ; 	// transfer data
	}
	// Note. Wait for the transfer to complete
	Start_ms = millis();
	while( millis()-Start_ms < TRANSMISSION_TIME);
	// Note. Wait for the incoming data.
	Start_ms = millis();
	while( millis()-Start_ms < TIMEOUT);

	while (serialDataAvail (fd))
	{ 
		RX_BUF[rx_count] = serialGetchar (fd);

		switch(rx_count)
		{
		case 0:
			if(RX_BUF[rx_count] == 0x16){rx_count++;}
			break;
		case 1:
			if(RX_BUF[rx_count] == 0x98){rx_count++;}
			else{rx_count=0;response_status = COM_ERROR;}
			break;
		case 2 ... 513:
			rx_count++;
			break;
		case 514:
			if(RX_BUF[rx_count] == 0x1A){rx_count++;}
			else{rx_count=0;response_status = COM_ERROR;}
			break;
		case 515:
			if(RX_BUF[rx_count] == 0x9C)
			{
				response_status = COM_OK;
								
				for(i=0; i<NUM_PIXEL; i++)
				{
					*au16regs = RX_BUF[2*i+2]<<8;
					*au16regs++ |= RX_BUF[2*i+3];
				}

			}
			else
			{
				response_status = COM_ERROR;
			}
			
			rx_count=0;   
			break;
		default: rx_count=0; break;

		} // end switch(rx_count)
	
	} //end while (serialDataAvail (fd))

	return response_status;
}

uint8_t highByte(uint16_t Data)
{
	return (uint8_t)((Data>>8)&0xFF);
}

uint8_t lowByte(uint16_t Data)
{
	return (uint8_t)(Data&0xFF);
}

void printarray(uint16_t icount, int16_t *p, uint16_t size) {
	if(size == 0) 
	{
		printf("\n");
     		return;
	}

	if(icount%8 == 7)
	{
		printf("%3dp(%3.1f)\n" , icount, *p/10.0);
	}
   	else
	{
		printf("%3dp(%3.1f) " , icount, *p/10.0);
	}

	printarray(++icount, ++p, --size );

}

